package system

import (
	"fmt"

	"code.fbi.h-da.de/danet/gnmi-target/handler"
	gnmitargetygot "code.fbi.h-da.de/danet/quant/goKMS/model"
	"code.fbi.h-da.de/danet/quant/goKMS/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

// MemoryHandler is the implementation of a gnmitarget.PathHandler.
type MemoryHandler struct {
	handler.DefaultPathHandler
	osClient osclient.Osclient
}

func NewMemoryHandler() *MemoryHandler {
	return &MemoryHandler{
		DefaultPathHandler: handler.DefaultPathHandler{
			Name: "openconfig-memory-handler",
			Paths: map[string]struct{}{
				"/system/memory/state": {},
			},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *MemoryHandler) Init(config *handler.Config, publishToSubsFunc func([]*gnmi.Notification) error) error {
	yh.Config = config
	yh.PublishToSubs = publishToSubsFunc

	_, err := yh.updateOrCreateMemory(yh.osClient)
	if err != nil {
		return err
	}
	return nil
}

func (yh *MemoryHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.Name)
	return nil
}

func (yh *MemoryHandler) updateOrCreateMemory(os osclient.Osclient) ([]*gnmi.Notification, error) {
	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return nil, err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return nil, fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	confSystem := newConfig.GetOrCreateSystem()

	if memory := confSystem.GetOrCreateMemory(); memory != nil {
		totalMemory, err := os.GetTotalMemory()
		if err != nil {
			return nil, fmt.Errorf("Failed to get the total memory; err: %w", err)
		}
		freeMemory, err := os.GetFreeMemory()
		if err != nil {
			return nil, fmt.Errorf("Failed to get the free memory; err: %w", err)
		}
		usedMemory, err := os.GetUsedMemory()
		if err != nil {
			return nil, fmt.Errorf("Failed to get the used memory; err: %w", err)
		}

		memory.GetOrCreateState().Physical = ygot.Uint64(totalMemory)
		memory.GetOrCreateState().Free = ygot.Uint64(freeMemory)
		memory.GetOrCreateState().Used = ygot.Uint64(usedMemory)
	}

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return nil, err
	}

	notifications, err := ygot.DiffWithAtomic(yh.Config.Data, newConfig)
	if err != nil {
		return nil, err
	}

	yh.Config.Data = newConfig

	return notifications, nil
}
