package restclient

import (
	"context"
	"net/http"

	restclient "code.fbi.h-da.de/danet/quant/etsi014/go/rest/etsi/client"
)

type ClientImpl struct {
	restClient *restclient.APIClient
}

func NewClientImpl(clientConf *restclient.Configuration) (*ClientImpl, error) {
	client := restclient.NewAPIClient(clientConf)

	return &ClientImpl{
		restClient: client,
	}, nil
}

func (c *ClientImpl) GetStatus(targetSAEID string) (*restclient.Status, *http.Response, error) {
	req := c.restClient.DefaultAPI.GetStatus(context.Background(), targetSAEID)
	return req.Execute()
}

func (c *ClientImpl) GetKey(targetSAEID string, number int64, size int64) (*restclient.KeyContainer, *http.Response, error) {
	req := c.restClient.DefaultAPI.GetKey(context.Background(), targetSAEID).Number(number).Size(size)
	return req.Execute()
}

func (c *ClientImpl) GetKeyPost(targetSAEID string, number int64, size int64, additionalTargetSAEIDs []string, extensionMandatory []map[string]string, extensionOptional []map[string]string) (*restclient.KeyContainer, *http.Response, error) {
	postReq := c.restClient.DefaultAPI.GetKeyPost(context.Background(), targetSAEID)
	keyReq := restclient.KeyRequest{
		Number: restclient.PtrInt64(number),
		Size:   restclient.PtrInt64(size),
	}
	if additionalTargetSAEIDs != nil {
		keyReq.AdditionalSlaveSAEIDs = additionalTargetSAEIDs
	}
	if extensionMandatory != nil {
		keyReq.ExtensionMandatory = extensionMandatory
	}
	if extensionOptional != nil {
		keyReq.ExtensionOptional = extensionOptional
	}

	return postReq.KeyRequest(keyReq).Execute()
}

func (c *ClientImpl) GetKeyWithId(targetSAEID, keyID string) (*restclient.KeyContainer, *http.Response, error) {
	req := c.restClient.DefaultAPI.GetKeyWithIds(context.Background(), targetSAEID).KeyID(keyID)

	return req.Execute()
}

func (c *ClientImpl) GetKeyWithIdPost(targetSAEID string, keyIDs []restclient.KeyIDsRequestKeyIDsInner) (*restclient.KeyContainer, *http.Response, error) {
	postReq := c.restClient.DefaultAPI.GetKeyWithIdsPost(context.Background(), targetSAEID)
	keyIDsReq := restclient.KeyIDsRequest{
		KeyIDs: keyIDs,
	}

	return postReq.KeyIDsRequest(keyIDsReq).Execute()
}
