package event

import "time"

type Topic int

const (
	PEER Topic = iota
	ROUTE
	QUANTUM_MODULE
	CREATE_ROUTE
	KEY_STORE
)

// Event ...
type Event interface {
	Topic() Topic
	Time() time.Time
}

type CreateRouteEvent struct {
	EventTopic   Topic
	Timestamp    time.Time
	KmsPathId    string
	RemoteUkmsId string
}

func NewCreateRouteEvent(kmsPathId, remoteUkmsId string) *CreateRouteEvent {
	return &CreateRouteEvent{
		EventTopic:   CREATE_ROUTE,
		Timestamp:    time.Now(),
		KmsPathId:    kmsPathId,
		RemoteUkmsId: remoteUkmsId,
	}
}

func (e *CreateRouteEvent) Topic() Topic {
	return e.EventTopic
}

func (e *CreateRouteEvent) Time() time.Time {
	return e.Timestamp
}

type PeerEvent struct {
	EventTopic Topic
	Timestamp  time.Time
	SocketStr  string
}

func NewPeerEvent(socket string) *PeerEvent {
	return &PeerEvent{
		EventTopic: PEER,
		Timestamp:  time.Now(),
		SocketStr:  socket,
	}
}

func (e *PeerEvent) Topic() Topic {
	return e.EventTopic
}

func (e *PeerEvent) Time() time.Time {
	return e.Timestamp
}

func (e *PeerEvent) Socket() string {
	return e.SocketStr
}

type RouteEvent struct {
	EventTopic Topic
	Timestamp  time.Time
}

func NewRouteEvent() *RouteEvent {
	return &RouteEvent{
		EventTopic: ROUTE,
		Timestamp:  time.Now(),
	}
}

func (e *RouteEvent) Topic() Topic {
	return e.EventTopic
}

func (e *RouteEvent) Time() time.Time {
	return e.Timestamp
}

type KeyStoreEvent struct {
	EventTopic Topic
	Timestamp  time.Time
}

func NewKeyStoreEvent() *KeyStoreEvent {
	return &KeyStoreEvent{
		EventTopic: KEY_STORE,
		Timestamp:  time.Now(),
	}
}

func (e *KeyStoreEvent) Topic() Topic {
	return e.EventTopic
}

func (e *KeyStoreEvent) Time() time.Time {
	return e.Timestamp
}
