#!/bin/bash

NEEDED_VERSION=$1
INSTALL_DIR="$HOME/go"

if [ -z "$NEEDED_VERSION" ]; then
  echo "Please provide the latest version of Go as an argument."
  exit 1
fi

# Check if the version of go defined in the file VERSION in $INSTALL_DIR is the same as the latest version
if [ -f "$INSTALL_DIR/go/VERSION" ]; then
  INSTALLED_VERSION=$(grep -oP '^go\d+\.\d+\.\d+' $INSTALL_DIR/go/VERSION | cut -c 3-)
  if [ "$INSTALLED_VERSION" == "$NEEDED_VERSION" ]; then
    echo "Go $NEEDED_VERSION is already installed."
    exit 0
  fi
fi

rm -rf $INSTALL_DIR

# Download the Go tarball
TARBALL_URL="https://go.dev/dl/go${NEEDED_VERSION}.linux-amd64.tar.gz"
wget $TARBALL_URL

# Extract the tarball to a directory within the user's home directory
mkdir -p $INSTALL_DIR
tar -C $INSTALL_DIR -xzf "go${NEEDED_VERSION}.linux-amd64.tar.gz"
rm "go${NEEDED_VERSION}.linux-amd64.tar.gz"

# Set up the environment variables
export PATH=$INSTALL_DIR/go/bin:$PATH
export GOROOT=$INSTALL_DIR/go
export GOPATH=$HOME/go_projects

go version

echo "Go has been installed successfully."
